<?php

namespace Drupal\panolens\Plugin\Field\FieldFormatter;

use Drupal\Core\Cache\Cache;
use Drupal\Core\Field\FieldItemListInterface;
use Drupal\image\Plugin\Field\FieldFormatter\ImageFormatter;

/**
 * Plugin implementation of the 'Image Panorama' formatter.
 *
 * @FieldFormatter(
 *   id = "panolens_image_panorama",
 *   label = @Translation("Image Panorama"),
 *   field_types = {
 *     "image"
 *   }
 * )
 */
class ImagePanoramaFormatter extends ImageFormatter {

  /**
   * {@inheritdoc}
   */
  public function viewElements(FieldItemListInterface $items, $langcode) {
    $elements = [];
    $files = $this->getEntitiesToView($items, $langcode);

    // Early opt-out if the field is empty.
    if (empty($files)) {
      return $elements;
    }

    $image_style_setting = $this->getSetting('image_style');

    // Collect cache tags to be added for each item in the field.
    $base_cache_tags = [];
    if (!empty($image_style_setting)) {
      $image_style = $this->imageStyleStorage->load($image_style_setting);
      $base_cache_tags = $image_style->getCacheTags();
    }

    foreach ($files as $delta => $file) {
      $cache_contexts = [];

      /** @var \Drupal\Core\File\FileUrlGenerator $file_url_generator */
      $file_url_generator = \Drupal::service('file_url_generator');
      $file_uri = $file_url_generator->generateString($file->getFileUri());

      $cache_contexts[] = 'url.site';
      $cache_tags = Cache::mergeTags($base_cache_tags, $file->getCacheTags());

      $elements[$delta] = [
        "#prefix" => '<div class="panolens-container">',
        '#markup' => '<div class="panolens-item" data-format="image-panorama" data-url="' . $file_uri . '"></div>',
        "#suffix" => '</div>',
        '#attached' => [
          'library' => [
            'panolens/image-panorama',
          ],
        ],
        '#cache' => [
          'tags' => $cache_tags,
          'contexts' => $cache_contexts,
        ],
      ];
    }
    return $elements;
  }

}
