<?php

namespace Drupal\panolens\Plugin\Field\FieldFormatter;

use Drupal\Core\Field\FieldItemListInterface;
use Drupal\file\Plugin\Field\FieldFormatter\FileVideoFormatter;

/**
 * Plugin implementation of the 'Video Panorama' formatter.
 *
 * @FieldFormatter(
 *   id = "panolens_video_panorama",
 *   label = @Translation("Video Panorama"),
 *   description = @Translation("Display the file using the Panolens.js library."),
 *   field_types = {
 *     "file"
 *   }
 * )
 */
class VideoPanoramaFormatter extends FileVideoFormatter {

  /**
   * {@inheritdoc}
   */
  public function viewElements(FieldItemListInterface $items, $langcode) {
    $elements = [];
    $source_files = $this->getSourceFiles($items, $langcode);

    // Early opt-out if the field is empty.
    if (empty($source_files)) {
      return $elements;
    }

    foreach ($source_files as $delta => $files) {
      $file = array_pop($files)['file'];
      $cache_contexts = [];

      /** @var \Drupal\Core\File\FileUrlGenerator $file_url_generator */
      $file_url_generator = \Drupal::service('file_url_generator');
      $file_uri = $file_url_generator->generateString($file->getFileUri());

      $cache_contexts[] = 'url.site';
      $cache_tags = $file->getCacheTags();

      $elements[$delta] = [
        "#prefix" => '<div class="panolens-container">',
        '#markup' => '<div class="panolens-item" data-format="video-panorama" data-url="' . $file_uri . '"></div>',
        "#suffix" => '</div>',
        '#attached' => [
          'library' => [
            'panolens/video-panorama',
          ],
        ],
        '#cache' => [
          'tags' => $cache_tags,
          'contexts' => $cache_contexts,
        ],
      ];
    }
    return $elements;
  }

}
